# SPARC

## Introduction

This is an implementation of SPARC, a graph-based semi-supervised learning method proposed in the following paper:

D Zhou, J He, H Yang, W Fan. SPARC: Self-Paced Network Representation for Few-Shot Rare Category Characterization, ACM SIGKDD Conference on Knowledge Discovery and Data Mining, KDD 2018.

Please cite the above paper if you use the datasets or code in this repo.

## Prepare the data
The demo dataset is preserved in the directory `data`, which includes
- `tx`, the feature vectors of the test instances,
- `ty`, the one-hot labels of the test instances,
- `test.index`, the indices of test instances in `graph`, for the inductive setting,
- `ally`, the labels for instances in `allx`.

The indices of test instances in `graph` for the transductive setting are from `#x` to `#x + #tx - 1`, with the same order as in `tx`.

You can use `cPickle.load(open(filename))` to load the numpy/scipy objects `x`, `y`, `tx`, `ty`, `allx`, `ally`, and `graph`. `test.index` is stored as a text file.

## Train the model

The input to SPARC contains:
- `x`, the feature vectors of the training instances,
- `y`, the one-hot labels of the training instances,
- `graph`, a `dict` in the format `{index: [index_of_neighbor_nodes]}`, where the neighbor nodes are organized as a list. The current version only supports binary graphs.

Let L be the number of training instances. The indices in `graph` from 0 to L - 1 must correspond to the training instances, with the same order as in `x`.


## Hyper-parameter tuning

Refer to `demo_SPARC.py` for the definition of different hyper-parameters (passed as arguments). Hyper-parameters are tuned by randomly shuffle the training/test split (i.e., randomly shuffling the indices in `x`, `y`, `tx`, `ty`, and `graph`). For the DIEL dataset, we tune the hyper-parameters on one of the ten runs, and then keep the same hyper-parameters for all the ten runs.

## Run the demo

We include the Citeseer dataset in the directory `data`, where the data structures needed are pickled.

To run SPARC on citeseer dataset,
```
python demo_SPARC.py
```

You can refer to `demo_SPARC.py` for example usages of our model.