
from scipy import sparse as sp
from trans_model import trans_model as model
import argparse
import cPickle
import numpy as np

DATASET = 'citeseer'

parser = argparse.ArgumentParser()
parser.add_argument('--learning_rate', help = 'learning rate for supervised loss', type = float, default = 0.1)
parser.add_argument('--embedding_size', help = 'embedding dimensions', type = int, default = 50)
parser.add_argument('--window_size', help = 'window size in random walk sequences', type = int, default = 3)
parser.add_argument('--path_size', help = 'length of random walk sequences', type = int, default = 10)
parser.add_argument('--batch_size', help = 'batch size for supervised loss', type = int, default = 200)
parser.add_argument('--g_batch_size', help = 'batch size for graph context loss', type = int, default = 200)
parser.add_argument('--g_sample_size', help = 'batch size for label context loss', type = int, default = 100)
parser.add_argument('--neg_samp', help = 'negative sampling rate; zero means using softmax', type = int, default = 10)
parser.add_argument('--g_learning_rate', help = 'learning rate for unsupervised loss', type = float, default = 1e-2)
parser.add_argument('--model_file', help = 'filename for saving models', type = str, default = 'trans.model')
parser.add_argument('--use_feature', help = 'whether use input features', type = bool, default = True)
parser.add_argument('--update_emb', help = 'whether update embedding when optimizing supervised loss', type = bool, default = True)
parser.add_argument('--layer_loss', help = 'whether incur loss on hidden layers', type = bool, default = True)
parser.add_argument('--cost_metric',help= 'cost sensitive metric', type = list, default = [1, 9, 0, 0, 0, 0])
parser.add_argument('--rare_ind',help= 'rare category indicator', type = int, default = 1)
args = parser.parse_args()

def comp_accu(tpy, ty):
    return (np.argmax(tpy, axis = 1) == np.argmax(ty, axis = 1)).sum() * 1.0 / tpy.shape[0]

def comp_recall(tpy, ty):
    ind = np.nonzero(ty[:,1]==1)
    count = ind[0].__len__()
    return tpy[ind,1].sum() *1.0 / count

def plot_embedding(X, title=None):
    import numpy as np
    import matplotlib.pyplot as plt
    from time import time
    from sklearn import datasets, manifold
    x_min, x_max = np.min(X, 0), np.max(X, 0)
    X = (X - x_min) / (x_max - x_min)

    plt.figure()
    ax = plt.subplot(111)
    for i in range(X.shape[0]):
        plt.text(X[i, 0], X[i, 1], str(digits.target[i]),
                 color=plt.cm.Set1(y[i] / 10.),
                 fontdict={'weight': 'bold', 'size': 9})
    plt.xticks([]), plt.yticks([])
    if title is not None:
        plt.title(title)

# load the data: x, y, tx, ty, graph
NAMES = ['x', 'y', 'tx', 'ty', 'graph']
OBJECTS = []
for i in range(len(NAMES)):
    OBJECTS.append(cPickle.load(open("data/trans.{}.{}".format(DATASET, NAMES[i]))))
x, y, tx, ty, graph = tuple(OBJECTS)

x_all = sp.vstack([x, tx])


# Manipulate imbalanced classes
for i in range(len(y)):
    if (y[i] == [0, 1, 0, 0, 0, 0]).all():
        y[i] = [0, 1, 0, 0, 0, 0]
    else:
        y[i] = [1, 0, 0, 0, 0, 0]

for i in range(len(ty)):
    if (ty[i] == [0, 1, 0, 0, 0, 0]).all():
        ty[i] = [0, 1, 0, 0, 0, 0]
    else:
        ty[i] = [1, 0, 0, 0, 0, 0]

# initialize the model
m = model(args)
m.add_data(x, y, tx, ty, graph, x_all)
m.build()
m.init_train_csl(init_iter_csl = 100)
m.init_train(init_iter_label = 2000, init_iter_graph = 70)
iter_cnt, max_accu, max_recall = 0, 0, 0
pace_num = 6
lamb = [0.1, 0.1]
theta = 1.1
alpha = 0.5


# train and test the model
for iter_pace in range(6):
    for iter_cnt in range(200):
        m.update_pace(lamb, alpha)
        m.step_train(max_iter = 1, iter_graph = 0.05, iter_rare = 0.03, iter_inst = 1, iter_label = 0)
        tpy = m.predict(tx)
        accu = comp_accu(tpy, ty)
        recall = comp_recall(tpy, ty)
        print iter_cnt, max_accu, max_recall
        if accu > max_accu:
            # store the model if better result is obtained
            m.store_params()
            max_accu = max(max_accu, accu)
        if recall > max_recall:
            # store the model if better result is obtained
            m.store_params()
            max_recall = max(max_recall, recall)

    # SPARC embedding over paces
    lamb = [theta*x for x in lamb]
    pace0_name = 'pace0_' + str(iter_pace)
    pace1_name = 'pace1_' + str(iter_pace)
    np.save(pace0_name, m.v0)
    np.save(pace1_name, m.v1)
    embed_RC = m.predict_embedding()
    name = './embed_RC' + str(iter_pace)
    np.save(name, embed_RC)
